﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using NoSqlCore.DBObject;

namespace NoSqlCore.Services.MongoDBServices
{
	public class Services : IServices, IDisposable
	{
		private ISession _session;


		public Services()
			: this(new MongoSession())
		{ 
		
		}

		public Services(ISession session)
		{
			this._session = session;
		}

		public void ClearCollection()
		{
			if (_session.All<MongoEvaluation>().Count() > 0)
			{
				_session.DeleteAll<MongoEvaluation>();
			}
		}

		public void AddEvaluation(Views.Evaluation.CreateData data)
		{
			MongoEvaluation evaluation = new MongoEvaluation();
			evaluation.Name = data.Name;
			evaluation.Type = data.Type;
			_session.Add(evaluation);
		}

		public void AddQuestionToEvaluation(string question, string evaluationId)
		{
			int intId = Convert.ToInt32(evaluationId);
			var evaluation = _session.Single<MongoEvaluation>(e => e.Id == intId);
			int rank = evaluation.QuestionList.Count() + 1;
			evaluation.QuestionList.Add(new Question { Description = question, Rank = rank });
			_session.Update(evaluation);
		}

		public List<Views.Evaluation.ListItem> GetEvaluationList()
		{
			var evaluationList = _session.All<MongoEvaluation>().OrderBy(t => t.Name);
			return evaluationList.Select(e => new Views.Evaluation.ListItem { Id = e.Id.ToString(), Name = e.Name, Type = e.Type.ToString() }).ToList();
		}

		public Views.Evaluation.ReadView GetReadView(string evaluationId)
		{
			int intId = Convert.ToInt32(evaluationId);
			var evaluation = _session.Single<MongoEvaluation>(e => e.Id == intId);
			Views.Evaluation.ReadView view = new Views.Evaluation.ReadView();
			view.EvaluationId = evaluation.Id.ToString();
			view.Name = evaluation.Name;
			view.QuestionList = evaluation.QuestionList.OrderBy(q => q.Rank)
					.Select(q => new NoSqlCore.Views.Evaluation.ReadView.GabaritCriteria { Description = q.Description, Rank = q.Rank }).ToList();
			return view;
		}

		public void DeleteQuestion(int rank, string evaluationId)
		{
			int intId = Convert.ToInt32(evaluationId);
			var evaluation = _session.Single<MongoEvaluation>(e => e.Id == intId);
			var question = evaluation.QuestionList.Single(q => q.Rank == rank);
			evaluation.QuestionList.Remove(question);

			evaluation.QuestionList
				.Where(q => q.Rank > rank).ToList()
				.ForEach(q => q.Rank--);

			_session.Update(evaluation);
		}

		public void DeleteEvaluation(string evaluationId)
		{
			int intId = Convert.ToInt32(evaluationId);
			_session.Delete<MongoEvaluation>(e => e.Id == intId);
		}

		public void Dispose()
		{
			if (_session != null)
			{
				_session.Dispose();
			}
		}
	}
}
