﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Raven.Client.Document;
using Raven.Client;
using NoSqlCore.DBObject;
using NoSqlCore.Views.Evaluation;

namespace NoSqlCore.Services.RavenDBSercices
{
	public class Services : IServices, IDisposable
	{

		private DocumentStore _documentStore;
		private IDocumentSession _session;

		public Services(DocumentStore documentStore)
			: this(documentStore, documentStore.OpenSession())
		{
			
		}

		public Services(DocumentStore documentStore, IDocumentSession session)
		{
			_documentStore = documentStore;
			_session = session;
		}

		public void ClearCollection()
		{
			var lst = _session.Query<RavenEvaluation>();
			foreach (var template in lst)
			{
				_session.Delete(template);
				_session.SaveChanges();
			}
		}

		public void AddEvaluation(Views.Evaluation.CreateData data)
		{
			RavenEvaluation evaluation = new RavenEvaluation();
			evaluation.Name = data.Name;
			evaluation.Type = data.Type;
			_session.Store(evaluation);
			_session.SaveChanges();
		}

		public void AddQuestionToEvaluation(string question, string evaluationId)
		{
			var evaluation = _session.Load<RavenEvaluation>(evaluationId);
			int rank = evaluation.QuestionList.Count() + 1;
			evaluation.QuestionList.Add(new Question { Description = question, Rank = rank });
			_session.SaveChanges();
		}

		public List<Views.Evaluation.ListItem> GetEvaluationList()
		{
			var evaluationList = _session.Query<RavenEvaluation>();
			return evaluationList.OrderBy(t => t.Name)
					.Select(t => new ListItem { Id = t.Id.ToString(), Name = t.Name, Type = t.Type.ToString() }).ToList();
		}

		public Views.Evaluation.ReadView GetReadView(string evalutionId)
		{
			ReadView view = new ReadView();
			var evaluation = _session.Load<RavenEvaluation>(evalutionId);
			view.EvaluationId = evaluation.Id.ToString();
			view.Name = evaluation.Name;
			view.QuestionList = evaluation.QuestionList.Select(c => new ReadView.GabaritCriteria { Rank = c.Rank, Description = c.Description }).ToList();
			return view;
		}

		public void DeleteQuestion(int rank, string evaluationId)
		{
			var evaluation = _session.Load<RavenEvaluation>(evaluationId);
			var question = evaluation.QuestionList.Single(r => r.Rank == rank);
			evaluation.QuestionList.Remove(question);
			evaluation.QuestionList
				.Where(q => q.Rank > rank).ToList()
				.ForEach(q => q.Rank--);

			_session.SaveChanges();
		}

		public void DeleteEvaluation(string evaluationId)
		{
			var evaluation = _session.Load<RavenEvaluation>(evaluationId);
			_session.Delete(evaluation);
			_session.SaveChanges();
		}

		public void Dispose()
		{
			if (_session != null)
			{
				_session.Dispose();
			}
		}
	}
}
