﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Raven.Client.Document;

namespace TestNoSqlCore
{
	[TestClass]
	public class RavenServices
	{
		private Raven.Client.Document.DocumentStore _documentStore;

		[TestInitialize]
		public void InitDocuimentStore()
		{
			_documentStore = new DocumentStore() { Url = "http://localhost:8080" };
			_documentStore.Initialize();

		}

		[TestMethod]
		public void FullTest()
		{
			using (NoSqlCore.Services.RavenDBSercices.Services services = new NoSqlCore.Services.RavenDBSercices.Services(_documentStore))
			{
				services.ClearCollection();
			}

			using (NoSqlCore.Services.RavenDBSercices.Services services = new NoSqlCore.Services.RavenDBSercices.Services(_documentStore))
			{
				services.AddEvaluation(new NoSqlCore.Views.Evaluation.CreateData { Name = "My First Evaluation", Type = NoSqlCore.Views.Evaluation.EvaluationType.Activity });
				services.AddEvaluation(new NoSqlCore.Views.Evaluation.CreateData { Name = "Another evaluation", Type = NoSqlCore.Views.Evaluation.EvaluationType.AutoEvalution });
			}

			using (NoSqlCore.Services.RavenDBSercices.Services services = new NoSqlCore.Services.RavenDBSercices.Services(_documentStore))
			{
				var list = services.GetEvaluationList();
				Assert.AreEqual(2, list.Count);
				Assert.IsTrue(list.Any(t => t.Type == NoSqlCore.Views.Evaluation.EvaluationType.Activity.ToString()));
				Assert.IsTrue(list.Any(t => t.Type == NoSqlCore.Views.Evaluation.EvaluationType.AutoEvalution.ToString()));
			}

			using (NoSqlCore.Services.RavenDBSercices.Services services = new NoSqlCore.Services.RavenDBSercices.Services(_documentStore))
			{
				var list = services.GetEvaluationList();
				var evaluation = list.Last();
				services.AddQuestionToEvaluation("this is a question", evaluation.Id);

				var evaluationReadView = services.GetReadView(evaluation.Id);
				Assert.AreEqual(1, evaluationReadView.QuestionList.Count);

				services.AddQuestionToEvaluation("question 2", evaluation.Id);
				services.AddQuestionToEvaluation("another question", evaluation.Id);
				services.AddQuestionToEvaluation("lorem ipsum doler", evaluation.Id);

				var evaluationReadView2 = services.GetReadView(evaluation.Id);
				Assert.AreEqual(4, evaluationReadView2.QuestionList.Count);
				Assert.IsTrue(evaluationReadView2.QuestionList.Any(c => c.Rank == 1));
				Assert.IsTrue(evaluationReadView2.QuestionList.Any(c => c.Rank == 2));
				Assert.IsTrue(evaluationReadView2.QuestionList.Any(c => c.Rank == 3));
				Assert.IsTrue(evaluationReadView2.QuestionList.Any(c => c.Rank == 4));
			}

			using (NoSqlCore.Services.RavenDBSercices.Services services = new NoSqlCore.Services.RavenDBSercices.Services(_documentStore))
			{
				var list = services.GetEvaluationList();
				var evaluation = list.Last();

				services.DeleteQuestion(2, evaluation.Id);
				var evaluationReadView = services.GetReadView(evaluation.Id);
				Assert.AreEqual(3, evaluationReadView.QuestionList.Count);
				Assert.IsTrue(evaluationReadView.QuestionList.Any(c => c.Rank == 1));
				Assert.IsTrue(evaluationReadView.QuestionList.Any(c => c.Rank == 2));
				Assert.IsTrue(evaluationReadView.QuestionList.Any(c => c.Rank == 3));
			}

			using (NoSqlCore.Services.RavenDBSercices.Services services = new NoSqlCore.Services.RavenDBSercices.Services(_documentStore))
			{
				var list = services.GetEvaluationList();
				var evaluation = list.Last();
				services.DeleteEvaluation(evaluation.Id);

				var list2 = services.GetEvaluationList();
				Assert.AreEqual(1, list2.Count);
			}
		}
	}
}
